-----------------------
-- SRAM Write with USB  
-----------------------

-- Description ---------------------------------------------------------------------------
-- SRAM Write with USB
--    USB Read1 -> USB Read2 -> USB Read3 -> USB Read4 -> SRAM Write(4byte) -> (back)
------------------------------------------------------------------------------------------

---- Notes ---------------------------------------------------------------------------------
-- 2007.01.25
--  initializerɓ]pړIŁB
--  Ƃ肠ǁASRSTM̃^C~OƂAvȁH
-- 2007.02.15
---- ςAf[^̓ɖ肠iSRAMʁj
---- ߂̋|CgH
--------------------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

library UNISIM;
use UNISIM.VCOMPONENTS.all;

entity SRAMUSB is
    port(
    	   CLKIN : in std_logic;
        LED : out std_logic_vector(7 downto 0);

        -- to SRAM
        SRAM_RW : out std_logic; -- read or write
        SRAM_IO : inout std_logic_vector(31 downto 0);  -- IO data
        SRAM_IOP : inout std_logic_vector(3 downto 0);  -- IO parity
        SRAM_AA : out std_logic_vector(19 downto 0);    -- address
        -- to SRAM(constant)
        SRAM_CLKM0,SRAM_CLKM1 : out std_logic;                      -- SRAM clock
        SRAM_BW : out std_logic_vector(3 downto 0);
        SRAM_CEH1X,SRAM_CEL1X,SRAM_CE2,SRAM_CE2X : out std_logic;
        SRAM_OEX,SRAM_CENX,SRAM_ADVLD : out std_logic;
        SRAM_LBOX : out std_logic;
        SRAM_ZZ : out std_logic;

        -- to USB
	   USB_TXEX : in std_logic;                         -- 1 when buffer is full
        USB_RSTX : out std_logic;                        -- USB reset
        USB_RDX : out std_logic;                         -- 0 when read
        USB_WR : out std_logic;                          -- 1 when write
        USB_RXFX : in std_logic;                         -- 1 when buffer is empty
        USB_DATA : inout std_logic_vector(7 downto 0));   -- USB data
end SRAMUSB;

architecture ARC of SRAMUSB is
    component ROC
        port(
            O : out std_logic);
    end component;

    component IBUFG
        port(
            I : in std_logic;
            O : out std_logic);
    end component;

    component SRAM
        port(
        CLK : in std_logic;                         -- clock 50MHz
        RESET : in std_logic;                       -- reset

        MODE : in std_logic;                       -- read/write mode
        ADDR : in std_logic_vector(19 downto 0);    -- read/write address
        AOUT : out std_logic_vector(19 downto 0);   -- current data address
        RDATA : out std_logic_vector(31 downto 0);  -- read data
        RP : out std_logic_vector(3 downto 0);      -- read parity
        WDATA : in std_logic_vector(31 downto 0);   -- write data
        WP : in std_logic_vector(3 downto 0);       -- write parity
        -- to SRAM
        RW : out std_logic; -- read or write
        IO : inout std_logic_vector(31 downto 0);  -- IO data
        IOP : inout std_logic_vector(3 downto 0);  -- IO parity
        AA : out std_logic_vector(19 downto 0);    -- address
        -- to SRAM(constant)
--        CLKM0,CLKM1 : out std_logic;                      -- SRAM clock
        BW : out std_logic_vector(3 downto 0);
        CEH1X,CEL1X,CE2,CE2X : out std_logic;
        OEX,CENX,ADVLD : out std_logic;
        LBOX : out std_logic;
        ZZ : out std_logic);
	end component;

    component USB
        port(
        CLK : in std_logic;                          -- clock 50MHz
        RESET : in std_logic;                        -- reset signal
        RMODE : in std_logic;                        -- 1 when read
        WMODE : in std_logic;                        -- 1 when write
        RDATA : out std_logic_vector(7 downto 0);    -- read data
        WDATA : in std_logic_vector(7 downto 0);     -- write data
        VALID : out std_logic;                       -- 1 when read is O.K.
        BUSY : out std_logic;
        -- to USB
        RDX : out std_logic;                         -- 0 when read
        WR : out std_logic;                          -- 1 when write
        TXEX : in std_logic;                         -- 1 when buffer is full
        RXFX : in std_logic;                         -- 1 when buffer is empty
        USBDATA : inout std_logic_vector(7 downto 0));  -- USB data
    end component;


    signal CLK : std_logic;
    signal RESET : std_logic;

    signal SRST : std_logic;
    signal SMODE : std_logic;
    signal SRDATA,SWDATA : std_logic_vector(31 downto 0);
    signal WBUF : std_logic_vector(31 downto 0);
    signal WBUF1,WBUF2,WBUF3 : std_logic_vector(7 downto 0);
    signal SADDR,SAOUT : std_logic_vector(19 downto 0);
    signal SRP,SWP : std_logic_vector(3 downto 0);

    signal RMODE : std_logic;
    signal WMODE : std_logic;
    signal RDATA : std_logic_vector(7 downto 0);
    signal WDATA : std_logic_vector(7 downto 0);
    signal VALID : std_logic;
    signal BUSY : std_logic;

    signal STATE : std_logic_vector(3 downto 0);
    signal COUNT : std_logic_vector(20 downto 0);

begin
    RC : ROC port map (
        O => RESET);

    -- Inner clk is "CLK"
    IBG : IBUFG port map (
        I => CLKIN,
        O => CLK);

    SRAM_INST : SRAM port map (
        CLK   => CLK,
        RESET => RESET,
        MODE  => SMODE,
        ADDR  => SADDR,
        AOUT  => SAOUT,
        RDATA => SRDATA,
        RP    => SRP,
        WDATA => SWDATA,
        WP    => SWP,
        RW    => SRAM_RW,
        IO    => SRAM_IO,
        IOP   => SRAM_IOP,
        AA    => SRAM_AA,
--        CLKM0  => SRAM_CLKM0,
--        CLKM1 => SRAM_CLKM1,
        BW    => SRAM_BW,
        CEH1X  => SRAM_CEH1X,
        CEL1X  => SRAM_CEL1X,
        CE2   => SRAM_CE2,
        CE2X  => SRAM_CE2X,
        OEX   => SRAM_OEX,
        CENX  => SRAM_CENX,
        ADVLD => SRAM_ADVLD,
        LBOX  => SRAM_LBOX,
        ZZ    => SRAM_ZZ);
  
    USB_INST : USB port map (
        CLK      => CLK,
        RESET    => RESET,
        RMODE    => RMODE,
        WMODE    => WMODE,
        RDATA    => RDATA,
        WDATA    => WDATA,
        VALID    => VALID,
        BUSY     => BUSY,
        RDX      => USB_RDX,
        WR       => USB_WR,
        TXEX     => USB_TXEX,
        RXFX     => USB_RXFX,
        USBDATA => USB_DATA);

	LED <= not (STATE & COUNT(19 downto 16));
    SWP <= "ZZZZ";
--    SWDATA <= WBUF when SRST = '1' else
--             (others => 'Z');
--    SADDR <= COUNT when SRST = '1' else
--            (others => 'Z');
    SMODE <= '0' when SRST = '1' else
             '1';
	SWDATA <= WBUF;
	SADDR <= COUNT(19 downto 0);


    SRAM_CLKM0 <= CLK;
    SRAM_CLKM1 <= CLK;

    process(CLK,RESET)
    begin
         if RESET = '1' then
            RMODE <= '0';
            WMODE <= '0';
            SRST <= '0';
            WDATA <= "00000000";
            STATE <= "1100";

		  USB_RSTX <= '0';

		  COUNT <= (others => '0');
          WBUF <= (others =>'0');

        elsif CLK'event and CLK = '1' then
	       USB_RSTX <= '1';
            case STATE is
		  	 -- prepare
		      when "1100" =>
                    if BUSY = '0' then
                        RMODE <= '1';
                        WMODE <= '0';
                        STATE <= STATE + 1;
                    else
                        STATE <= "1100";
                    end if;
                when "1101" =>
                    if VALID = '1' then
                        RMODE <= '0';
                        STATE <= STATE + 1;
                    else
                        STATE <= "1101";
                    end if;
		  	 when "1110" =>
				if BUSY = '0' then
					WMODE <= '1';
					STATE <= STATE + 1;
				else
					STATE <= "1110";
				end if;
			 when "1111" =>
				if VALID = '1' then
					WMODE <= '0';
					STATE <= "0000";
				else
					STATE <= "1111";
				end if;
			-- main cycle
                when "0000" =>
			 	if COUNT(20) = '1' then
					STATE <= "1010";
                    elsif BUSY = '0' then  --- USB read 1-1
                        RMODE <= '1';
                        STATE <= STATE + 1;
                    else
                        STATE <= "0000";
                    end if;
                when "0001" =>
                    if VALID = '1' then --- USB read 1-2
                        RMODE <= '0';
                        WBUF1 <= RDATA;
                        STATE <= STATE + 1;
                    else
                        STATE <= "0001";
                    end if;
                when "0010" =>
                    if BUSY = '0' then  --- USB read 2-1
                        RMODE <= '1';
                        STATE <= STATE + 1;
                    else
                        STATE <= "0010";
                    end if;
                when "0011" =>
                    if VALID = '1' then --- USB read 2-2
                        RMODE <= '0';
                        WBUF2 <= RDATA;
                        STATE <= STATE + 1;
                    else
                        STATE <= "0011";
                    end if;
                when "0100" =>
                    if BUSY = '0' then  --- USB read 3-1
                        RMODE <= '1';
                        STATE <= STATE + 1;
                    else
                        STATE <= "0100";
                    end if;
                when "0101" =>
                    if VALID = '1' then --- USB read 3-2
                        RMODE <= '0';
                        WBUF3 <= RDATA;
                        STATE <= STATE + 1;
                    else
                        STATE <= "0101";
                    end if;
                when "0110" =>
                    if BUSY = '0' then  --- USB read 4-1
                        RMODE <= '1';
                        STATE <= STATE + 1;
                    else
                        STATE <= "0110";
                    end if;
                when "0111" =>
                    if VALID = '1' then --- USB read 4-2
                        RMODE <= '0';
  			         WBUF <= WBUF1 & WBUF2 & WBUF3 & RDATA;
                        SRST <= '1';
                        STATE <= STATE + 1;
                    else
                        STATE <= "0111";
                    end if;
                when "1000" =>
                    STATE <= STATE + 1;
                when "1001" =>
			     SRST <= '0';
                    COUNT <= COUNT + 1;
                    STATE <= "0000";
			when "1010" =>
				STATE <= "1010";
                when others =>
                    WMODE <= '0';
                    RMODE <= '0';
                    SRST <= '0';
                    STATE <= "0000";
                end case;
        end if;
    end process;

end ARC;
